Package: imagine
Type: Package
Title: IMAGing engINEs, Tools for Application of Image Filters to Data
        Matrices
Version: 2.1.4
Date: 2026-05-13
URL: https://github.com/LuisLauM/imagine
BugReports: https://github.com/LuisLauM/imagine/issues
Authors@R: c(person("Wencheng", "Lau-Medrano", role = c("aut", "cre"), email = "luis.laum@protonmail.com"))
Maintainer: Wencheng Lau-Medrano <luis.laum@protonmail.com>
Description: Provides fast application of image filters to data matrices,
    using R and C++ algorithms.
License: GPL (>= 2)
LazyData: TRUE
Depends: R (>= 4.1.0)
Imports: cli, Rcpp, RcppArmadillo
LinkingTo: Rcpp, RcppArmadillo
Suggests: knitr, rmarkdown, quarto
VignetteBuilder: quarto, knitr
Encoding: UTF-8
RoxygenNote: 8.0.0
NeedsCompilation: yes
Packaged: 2026-05-13 16:16:05 UTC; afipdbl
Author: Wencheng Lau-Medrano [aut, cre]
Repository: CRAN
Date/Publication: 2026-05-14 14:20:18 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-14 15:36:38 UTC; unix
Archs: imagine.so.dSYM
