# Generated by roxygen2: do not edit by hand

export(agenbagFilters)
export(contextualMF)
export(convolution2D)
export(convolutionMedian)
export(convolutionQuantile)
export(meanFilter)
export(medianFilter)
export(quantileFilter)
importFrom(Rcpp,evalCpp)
importFrom(RcppArmadillo,armadillo_version)
importFrom(cli,cli_abort)
importFrom(cli,cli_warn)
useDynLib(imagine, .registration = TRUE)
