#' @title IMAGing engINE, Tools for application of image filters to data matrices
#' @description Provides fast application of image filters to data matrices,
#' using R and C++ algorithms.
#' @details This package uses C++ algorithms called 'engines'. More details are
#' shown in the vignette.
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom cli cli_abort
#' @importFrom cli cli_warn
## usethis namespace: end
NULL
